"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getEvents = exports.persist = void 0;
const client_dynamodb_1 = require("@aws-sdk/client-dynamodb");
const util_dynamodb_1 = require("@aws-sdk/util-dynamodb");
const optional_1 = require("@othree.io/optional");
const persist = (dynamoDb, configuration) => (event) => __awaiter(void 0, void 0, void 0, function* () {
    return (0, optional_1.TryAsync)(() => __awaiter(void 0, void 0, void 0, function* () {
        const command = new client_dynamodb_1.PutItemCommand({
            Item: (0, util_dynamodb_1.marshall)(event, { removeUndefinedValues: true }),
            TableName: configuration.TableName
        });
        yield dynamoDb.send(command);
        return event;
    }));
});
exports.persist = persist;
const getEvents = (dynamoDb, configuration) => (contextId) => __awaiter(void 0, void 0, void 0, function* () {
    return (0, optional_1.TryAsync)(() => __awaiter(void 0, void 0, void 0, function* () {
        const command = new client_dynamodb_1.QueryCommand({
            TableName: configuration.TableName,
            KeyConditionExpression: 'contextId = :contextId',
            ExpressionAttributeValues: {
                ':contextId': { 'S': contextId }
            },
            Limit: configuration.SnapshotFrequency,
            ScanIndexForward: false
        });
        const result = yield dynamoDb.send(command);
        if (result.Items) {
            return (result.Items.map(_ => (0, util_dynamodb_1.unmarshall)(_)));
        }
        return [];
    }));
});
exports.getEvents = getEvents;
//# sourceMappingURL=event-source.js.map